"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.APIAuthClient = void 0;
const util_1 = require("./util");
class APIAuthClient {
    constructor({ clientId, sharedSecret, authHost }, fetcher, authTicketCache, isInternal) {
        if (!clientId || !sharedSecret || !authHost) {
            throw new Error("Kibo API Auth client requires a clientId, sharedSecret, and authUrl");
        }
        if (!fetcher) {
            throw new Error("Kibo API Auth client requires a Fetch API implementation");
        }
        this._clientId = clientId;
        this._sharedSecret = sharedSecret;
        this._authHost = util_1.addProtocolToHost(authHost);
        this._fetcher = fetcher;
        this._isInternal = isInternal;
        if (authTicketCache) {
            this._authTicketCache = authTicketCache;
        }
    }
    _buildFetchOptions(body = {}) {
        return Object.assign({ method: "POST", headers: {
                "Content-Type": "application/json",
            }, body: JSON.stringify(body) }, util_1.getProxyAgent());
    }
    async _fetchAuthTicket(url, options) {
        // fetch app auth ticket
        const response = await this._fetcher(url, options);
        if ((response === null || response === void 0 ? void 0 : response.status) >= 500) {
            throw new Error(response === null || response === void 0 ? void 0 : response.statusText);
        }
        const authTicket = (await response.json());
        if (authTicket === null || authTicket === void 0 ? void 0 : authTicket.errorCode) {
            throw new Error(authTicket.message);
        }
        // set expiration time in ms on auth ticket
        authTicket.expires_at = util_1.calculateTicketExpiration(authTicket);
        return authTicket;
    }
    async authenticate() {
        var _a;
        // create oauth fetch options
        const options = this._buildFetchOptions({
            client_id: this._clientId,
            client_secret: this._sharedSecret,
            grant_type: "client_credentials",
        });
        const path = this._isInternal ? `/platform/applications/internal/access-tokens/oauth` : `/api/platform/applications/authtickets/oauth`;
        // perform authentication
        const authTicket = await this._fetchAuthTicket(`${this._authHost}${path}`, options);
        // set authentication ticket on next server runtime object
        (_a = this._authTicketCache) === null || _a === void 0 ? void 0 : _a.setAuthTicket(authTicket);
        return authTicket;
    }
    async refreshTicket(kiboAuthTicket) {
        var _a;
        // create oauth refresh fetch options
        const options = this._buildFetchOptions({
            client_id: this._clientId,
            client_secret: this._sharedSecret,
            grant_type: "client_credentials",
            refresh_token: kiboAuthTicket === null || kiboAuthTicket === void 0 ? void 0 : kiboAuthTicket.refresh_token,
        });
        const path = this._isInternal ? `/api/platform/applications/internal/access-tokens/oauth` : `/api/platform/applications/authtickets/oauth`;
        // perform auth ticket refresh
        const refreshedTicket = await this._fetchAuthTicket(`${this._authHost}${path}`, options);
        // set authentication ticket on next server runtime object
        (_a = this._authTicketCache) === null || _a === void 0 ? void 0 : _a.setAuthTicket(refreshedTicket);
        return refreshedTicket;
    }
    async getAccessToken() {
        var _a;
        // get current Kibo API auth ticket
        try {
            let authTicket = await ((_a = this._authTicketCache) === null || _a === void 0 ? void 0 : _a.getAuthTicket());
            // if no current ticket, perform auth
            // or if ticket expired, refresh auth
            if (!authTicket) {
                authTicket = await this.authenticate();
            }
            else if (authTicket.expires_at < Date.now()) {
                authTicket = await this.refreshTicket(authTicket);
            }
            return authTicket === null || authTicket === void 0 ? void 0 : authTicket.access_token;
        }
        catch (error) {
            throw new Error(error.message);
        }
    }
}
exports.APIAuthClient = APIAuthClient;
//# sourceMappingURL=index.js.map